/*
 * Copyright 2012 Smallcrew, Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package fr.smallcrew.test;

import com.jayway.restassured.path.json.JsonPath;
import com.jayway.restassured.path.xml.XmlPath;
import com.jayway.restassured.response.Response;
import org.springframework.http.MediaType;

import java.util.List;

import static org.fest.assertions.Assertions.assertThat;

public class ResponsePath {
  public static final String ROOT = "response";

  private final XmlPath xmlPath;
  private final JsonPath jsonPath;

  private ResponsePath(Object impl, String root) {
    if (impl instanceof XmlPath) {
      xmlPath = (XmlPath) impl;
      if (root == null) {
        root = ROOT;
      }
      xmlPath.setRoot(root);
      jsonPath = null;
    } else {
      jsonPath = (JsonPath) impl;
      xmlPath = null;
    }
  }

  public static ResponsePath from(Response response, MediaType mediaType) {
    return from(response, mediaType, null);
  }

  public static ResponsePath from(Response response, MediaType mediaType, String rootPath) {

    if (mediaType.equals(MediaType.APPLICATION_JSON)) {
      return new ResponsePath(response.jsonPath(), null);
    } else if (mediaType.equals(MediaType.APPLICATION_XML)) {
      return new ResponsePath(response.xmlPath(), rootPath);
    }
    throw new IllegalArgumentException("Unknown media type " + response.getContentType());
  }

  public boolean getBoolean(String path) {
    if (xmlPath != null) {
      return xmlPath.getBoolean(path);
    }
    return jsonPath.getBoolean(path);
  }

  public int getInt(String path) {
    if (xmlPath != null) {
      return xmlPath.getInt(path);
    }
    return jsonPath.getInt(path);
  }

  public String getString(String path) {
    if (xmlPath != null) {
      return xmlPath.getString(path);
    }
    return jsonPath.getString(path);
  }

  public <T> List<T> getList(String path) {
    if (xmlPath != null) {
      return xmlPath.getList(path);
    }
    return jsonPath.getList(path);
  }

  public void isTrue(String path) {
    assertThat(getBoolean(path)).isTrue();
  }

  public void isFalse(String path) {
    assertThat(getBoolean(path)).isFalse();
  }
}
